@@ -13,11 +13,14 @@
 //! use rad::Processor;
 //! use rad::MacroType;
 //! use rad::AuthType;
+//! use rad::CommentType;
 //! use std::path::Path;
 //! 
 //! // Builder
 //! let mut processor = Processor::new()
-//!     .comment(true)                                       // Use comments
+//!     .comment(CommentType::Start)                         // Use comment
+//!     .custom_macro_char('~')                              // use custom macro character
+//!     .custom_comment_char('#')                            // use custom comment character
 //!     .purge(true)                                         // Purge undefined macro
 //!     .greedy(true)                                        // Makes all macro greedy
 //!     .silent(true)                                        // Silents all warnings
@@ -38,6 +41,9 @@
 //!     .interactive(true)                                   // Use interactive mode
 //!     // Create unreferenced instance
 //!     .build(); 
+//!
+//! // Comment char and macro char cannot be same 
+//! // Unallowed pattern for the characters are [a-zA-Z1-9\\_\*\^\|\+\(\)=,]
 //! 
 //! // Use Processor::empty() instead of Processor::new()
 //! // if you don't want any default macros
@@ -59,9 +65,21 @@
 //!     })
 //! );
 //!
+//! // Register a hook macro
+//! // Trigger and execution macro should be defined otherwise
+//! processor.register_hook(
+//! 	HookType::Macro,            // Macro type
+//! 	trigger_macro,              // Macro that triggers
+//! 	hook_div,                   // Macro to be executed
+//! 	1,    						// target count
+//! 	false 						// Resetable
+//! )
 //! 
 //! // Add custom rules(in order of "name, args, body") 
 //! processor.add_custom_rules(vec![("test","a_src a_link","$a_src() -> $a_link()")]);
+//!
+//! // Add custom rules without any arguments
+//! processor.add_static_rules(vec![("test","TEST"),("lul","kekw")])
 //! 
 //! // Process with inputs
 //! // This prints to desginated write destinations
@@ -80,6 +98,8 @@
 
 use crate::auth::{AuthType, AuthFlags, AuthState};
 #[cfg(feature = "debug")]
+use crate::models::DiffOption;
+#[cfg(feature = "debug")]
 use crate::debugger::DebugSwitch;
 #[cfg(feature = "debug")]
 use std::io::Read;
@@ -93,16 +113,21 @@ use crate::error::RadError;
 use crate::logger::{Logger, LoggerLines};
 #[cfg(feature = "debug")]
 use crate::debugger::Debugger;
-use crate::models::{
-    MacroMap, MacroRule, RuleFile, 
-    UnbalancedChecker, WriteOption, 
-    MacroFragment
-};
+use crate::models::{CommentType, MacroFragment, MacroMap, MacroRule, RuleFile, UnbalancedChecker, WriteOption};
+#[cfg(feature = "hook")]
+use crate::hookmap::{HookMap, HookType};
 use crate::utils::Utils;
-use crate::consts::*;
+use crate::{RadResult, consts::*};
 use crate::lexor::*;
 use crate::define_parser::DefineParser;
 use crate::arg_parser::{ArgParser, GreedyState};
+use regex::Regex;
+use lazy_static::lazy_static;
+
+lazy_static! {
+    // Source : https://stackoverflow.com/questions/17564088/how-to-form-a-regex-to-recognize-correct-declaration-of-variable-names/17564142
+    static ref MAC_NAME : Regex = Regex::new(r#"^[_a-zA-Z]\w*$"#).expect("Failed to create regex expression");
+}
 
 // Methods of processor consists of multiple sections followed as
 // <BUILDER>            -> Builder pattern related
@@ -130,11 +155,13 @@ pub(crate) struct ProcessorState {
     pub redirect: bool,
     pub sandbox: bool,
     pub strict: bool,
-    pub use_comment: bool,
+    pub comment_type: CommentType,
     // Temp target needs to save both path and file
     // because file doesn't necessarily have path. 
     // Especially in unix, this is not so an unique case
     pub temp_target: (PathBuf,File), 
+    pub comment_char : Option<char>,
+    pub macro_char : Option<char>,
 }
 
 impl ProcessorState {
@@ -148,11 +175,13 @@ impl ProcessorState {
             redirect: false,
             purge: false,
             strict: true,
-            use_comment: false,
+            comment_type: CommentType::None,
             nopanic: false,
             sandbox : false,
             always_greedy: false,
             temp_target,
+            comment_char: None,
+            macro_char: None,
         }
     }
 }
@@ -161,6 +190,8 @@ impl ProcessorState {
 pub struct Processor{
     map: MacroMap,
     closure_map: ClosureMap,
+    #[cfg(feature = "hook")]
+    pub(crate) hook_map: HookMap,
     defparser: DefineParser,
     write_option: WriteOption,
     logger: Logger,
@@ -214,6 +245,8 @@ impl Processor {
         Self {
             map,
             closure_map: ClosureMap::new(),
+            #[cfg(feature = "hook")]
+            hook_map: HookMap::new(),
             write_option: WriteOption::Terminal,
             defparser: DefineParser::new(),
             logger,
@@ -261,6 +294,31 @@ impl Processor {
         Ok(self)
     }
 
+    /// Custom comment character
+    pub fn custom_comment_char(&mut self, character: char) -> Result<&mut Self, RadError> {
+        // check if unallowed character
+        if UNALLOWED_CHARS.is_match(&character.to_string()) {
+            return Err(RadError::UnallowedChar(format!("\"{}\" is not allowed", character)));
+        } else if self.get_macro_char() == character {
+            // macro char and comment char should not be equal
+            return Err(RadError::UnallowedChar(format!("\"{}\" is already defined for macro character", character)));
+        }
+        self.state.comment_char.replace(character);
+        Ok(self)
+    }
+
+    /// Custom macro character
+    pub fn custom_macro_char(&mut self, character: char) -> Result<&mut Self, RadError> {
+        if UNALLOWED_CHARS.is_match(&character.to_string()) {
+            return Err(RadError::UnallowedChar(format!("\"{}\" is not allowed", character)));
+        } else if self.get_comment_char() == character {
+            // macro char and comment char should not be equal
+            return Err(RadError::UnallowedChar(format!("\"{}\" is already defined for comment character", character)));
+        }
+        self.state.macro_char.replace(character);
+        Ok(self)
+    }
+
     /// Use unix line ending instead of operating system's default one
     pub fn unix_new_line(&mut self, use_unix_new_line: bool) -> &mut Self {
         if use_unix_new_line {
@@ -288,8 +346,18 @@ impl Processor {
     }
 
     /// Use comment
+    ///
+    /// This is deprecated and will be removed in 2.0 version.
+    /// Use set_comment_type instead.
+    #[deprecated(since = "1.2", note = "Comment is deprecated and will be removed in 2.0 version.")]
     pub fn comment(&mut self, comment: bool) -> &mut Self {
-        self.state.use_comment = comment; 
+        if comment { self.state.comment_type = CommentType::Start; }
+        self
+    }
+
+    /// Set comment type
+    pub fn set_comment_type(&mut self, comment_type: CommentType) -> &mut Self {
+        self.state.comment_type = comment_type;
         self
     }
 
@@ -337,8 +405,8 @@ impl Processor {
 
     /// Add diff option
     #[cfg(feature = "debug")]
-    pub fn diff(&mut self, diff: bool) -> Result<&mut Self, RadError> {
-        if diff { self.debugger.enable_diff()?; }
+    pub fn diff(&mut self, diff: DiffOption) -> Result<&mut Self, RadError> {
+        self.debugger.enable_diff(diff)?; 
         Ok(self)
     }
 
@@ -481,8 +549,12 @@ impl Processor {
     /// ```rust
     /// processor.add_custom_rules(vec![("macro_name","macro_arg1 macro_arg2","macro_body=$macro_arg1()")]);
     /// ```
-    pub fn add_custom_rules(&mut self, rules: Vec<(&str,&str,&str)>) {
+    pub fn add_custom_rules(&mut self, rules: Vec<(impl AsRef<str>,&str,&str)>) -> Result<(), RadError> {
         for (name,args,body) in rules {
+            let name = name.as_ref();
+            if !MAC_NAME.is_match(name) {
+                return Err(RadError::InvalidMacroName(format!("Name : \"{}\" is not a valid macro name", name)));
+            }
             self.map.custom.insert(
                 name.to_owned(), 
                 MacroRule { 
@@ -492,8 +564,65 @@ impl Processor {
                 }
             );
         }
+        Ok(())
     }
 
+    /// Add static rules without builder pattern
+    ///
+    /// # Args
+    ///
+    /// The order of argument is "name, body"
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// processor.add_custom_rules(vec![("macro_name","Macro body without arguments")]);
+    /// ```
+    pub fn add_static_rules(&mut self, rules: Vec<(impl AsRef<str>,&str)>) -> RadResult<()> {
+        for (name,body) in rules {
+            let name = name.as_ref();
+            if !MAC_NAME.is_match(name) {
+                return Err(RadError::InvalidMacroName(format!("Name : \"{}\" is not a valid macro name", name)));
+            }
+            self.map.custom.insert(
+                name.to_owned(), 
+                MacroRule { 
+                    name: name.to_owned(),
+                    args: vec![],
+                    body: body.to_owned()
+            }
+            );
+        }
+        Ok(())
+    }
+
+    #[cfg(feature = "hook")]
+    /// Register a hook
+    pub fn register_hook(&mut self, hook_type: HookType, target_macro: &str, invoke_macro: &str, target_count: usize, resetable: bool) -> RadResult<()> {
+        // Check target macro is empty
+        if target_macro.len() == 0 {
+            return Err(RadError::InvalidMacroName(format!("Cannot register hook for macro \"{}\"",target_macro)));
+        }
+
+        // Check invoke macro is empty
+        if invoke_macro.len() == 0 {
+            return Err(RadError::InvalidMacroName(format!("Cannot register hook which invokes a macro \"{}\"",target_macro)));
+        }
+        self.hook_map.add_hook(hook_type, target_macro, invoke_macro, target_count, resetable)?;
+        Ok(())
+    }
+
+    #[cfg(feature = "hook")]
+    /// Deregister 
+    pub fn deregister_hook(&mut self, hook_type: HookType, target_macro: &str) -> RadResult<()> {
+        // Check target macro is empty
+        if target_macro.len() == 0 {
+            return Err(RadError::InvalidMacroName(format!("Cannot deregister hook for macro \"{}\"",target_macro)));
+        }
+
+        self.hook_map.del_hook(hook_type, target_macro)?;
+        Ok(())
+    }
 
     /// Read from string
     pub fn from_string(&mut self, content: &str) -> Result<(), RadError> {
@@ -556,7 +685,7 @@ impl Processor {
     /// Internal method for processing buffers line by line
     fn from_buffer(&mut self,buffer: &mut impl std::io::BufRead, backup: Option<SandboxBackup>, use_container: bool) -> Result<Option<String>, RadError> {
         let mut line_iter = Utils::full_lines(buffer).peekable();
-        let mut lexor = Lexor::new();
+        let mut lexor = Lexor::new(self.get_macro_char(), self.get_comment_char(), &self.state.comment_type);
         let mut frag = MacroFragment::new();
 
         // Container can be used when file include is nested inside macro definition
@@ -616,7 +745,7 @@ impl Processor {
         } // Loop end
 
         // Recover
-        if let Some(backup) = backup { self.recover(backup); self.state.sandbox = false; }
+        if let Some(backup) = backup { self.recover(backup)?; self.state.sandbox = false; }
 
         if use_container {
             Ok(cont)
@@ -714,7 +843,7 @@ impl Processor {
 
     /// Parse chunk args by separating it into lines which implements BufRead
     pub(crate) fn parse_chunk_args(&mut self, level: usize, _caller: &str, chunk: &str) -> Result<String, RadError> {
-        let mut lexor = Lexor::new();
+        let mut lexor = Lexor::new(self.get_macro_char(), self.get_comment_char(), &self.state.comment_type);
         let mut frag = MacroFragment::new();
         let mut result = String::new();
         let backup = self.logger.backup_lines();
@@ -748,7 +877,7 @@ impl Processor {
     /// In contrast to parse_chunk_lines, parse_chunk doesn't create lines iterator but parses the
     /// chunk as a single entity or line.
     fn parse_chunk_body(&mut self, level: usize, caller: &str, chunk: &str) -> Result<String, RadError> {
-        let mut lexor = Lexor::new();
+        let mut lexor = Lexor::new(self.get_macro_char(), self.get_comment_char(), &self.state.comment_type);
         let mut frag = MacroFragment::new();
         let backup = self.logger.backup_lines();
 
@@ -775,17 +904,21 @@ impl Processor {
 
         // Check comment line
         // If it is a comment then return nothing and write nothing
-        if self.state.use_comment && line.starts_with(COMMENT_CHAR) {
-            return Ok(String::new());
+        if self.state.comment_type != CommentType::None 
+            && line.starts_with(self.get_comment_char()) {
+                return Ok(String::new());
         }
 
         for ch in line.chars() {
-
             self.logger.add_char_number();
 
             let lex_result = lexor.lex(ch)?;
             // Either add character to remainder or fragments
             match lex_result {
+                LexResult::CommentExit => {
+                    self.lex_branch_comment_exit(frag, &mut remainder);
+                    return Ok(remainder);
+                }
                 LexResult::Discard => (),
                 LexResult::Ignore => frag.whole_string.push(ch),
                 // If given result is literal
@@ -799,7 +932,7 @@ impl Processor {
                     // This restart frags
                     remainder.push_str(&frag.whole_string);
                     frag.clear();
-                    frag.whole_string.push('$');
+                    frag.whole_string.push(self.get_macro_char());
                 },
                 LexResult::EmptyName => {
                     self.lex_branch_empty_name(ch, frag, &mut remainder, lexor);
@@ -818,6 +951,31 @@ impl Processor {
                     self.lex_branch_exit_frag(ch,frag,&mut remainder);
                 }
             }
+
+            // Character hook macro evaluation
+            // This only works on plain texts with level 0 
+            #[cfg(feature = "hook")]
+            if frag.is_empty() && level == 0 {
+                if let Some(mac_name) = self.hook_map.add_char_count(ch) {
+                    // If no fragment information is saved then
+                    let mut hook_frag = MacroFragment::new();
+                    hook_frag.name = mac_name;
+                    let mut hook_mainder = String::new();
+                    let mut hook_lexor = Lexor::new(self.get_macro_char(), self.get_comment_char(), &self.state.comment_type);
+                    self.lex_branch_end_invoke(
+                        &mut hook_lexor,
+                        &mut hook_frag,
+                        &mut hook_mainder,
+                        level,
+                        &frag.name
+                    )?;
+
+                    // If there is content to add
+                    if hook_mainder.len() != 0 {
+                        remainder.push_str(&hook_mainder);
+                    } 
+                } // End if let of macro name
+            }
         } // End Character iteration
         Ok(remainder)
     }
@@ -827,6 +985,7 @@ impl Processor {
     /// Evaluate detected macro usage
     ///
     /// Evaluation order is followed
+    /// - Keyword macro
     /// - Local bound macro
     /// - Custom macro
     /// - Basic macro
@@ -976,6 +1135,11 @@ impl Processor {
     // <LEX>
     // Start of lex branch methods
     // These are parse's sub methods for eaiser reading
+    fn lex_branch_comment_exit(&mut self, frag: &mut MacroFragment, remainder: &mut String) {
+        remainder.push_str(&frag.whole_string);
+        remainder.push_str(&self.state.newline);
+        frag.clear();
+    }
     fn lex_branch_literal(&mut self, ch: char,frag: &mut MacroFragment, remainder: &mut String, cursor: Cursor) {
         match cursor {
             // Exit frag
@@ -1078,6 +1242,9 @@ impl Processor {
         Ok(())
     }
 
+    // TODO
+    // This should be renamed because it is now, not only used by lex branches
+    // but also used by external logic functions such as hook macro executions.
     fn lex_branch_end_invoke(&mut self,lexor: &mut Lexor, frag: &mut MacroFragment, remainder: &mut String, level: usize, caller: &str) -> Result<(), RadError> {
         // Name is empty
         if frag.name.len() == 0 {
@@ -1137,11 +1304,9 @@ impl Processor {
         }
     }
 
-    // Level is needed for feature debug codes
+    // Level is needed for feature debug & hook codes
     fn lex_branch_end_frag_eval_result_ok(&mut self, variant : EvalResult, frag: &mut MacroFragment, remainder: &mut String, lexor : &mut Lexor, level: usize) -> Result<(), RadError> {
         match variant {
-            // else it is ok to proceed.
-            // thus it is safe to unwrap it
             EvalResult::Eval(content) => {
 
                 // Debug
@@ -1158,6 +1323,8 @@ impl Processor {
                 if let None = content {
                     lexor.escape_next_newline();
                 } else {
+                    // else it is ok to proceed.
+                    // thus it is safe to unwrap it
                     let mut content = content.unwrap();
                     if frag.trimmed {
                         content = Utils::trim(&content);
@@ -1165,8 +1332,35 @@ impl Processor {
                     if frag.yield_literal {
                         content = format!("\\*{}*\\", content);
                     }
+
+                    // TODO
+                    // Check unpredicted duplicate hook
+                    // Macro hook check
+                    #[cfg(feature = "hook")]
+                    if let Some(mac_name) = self.hook_map.add_macro_count(&frag.name) {
+                        let mut hook_frag = MacroFragment::new();
+                        hook_frag.name = mac_name;
+                        let mut hook_mainder = String::new();
+                        let mut hook_lexor = Lexor::new(self.get_macro_char(), self.get_comment_char(), &self.state.comment_type);
+                        self.lex_branch_end_invoke(
+                            &mut hook_lexor,
+                            &mut hook_frag,
+                            &mut hook_mainder,
+                            level,
+                            &frag.name
+                        )?;
+
+                        // Add hook remainder into current content
+                        // which will be added to main remainder
+                        if hook_mainder.len() != 0 {
+                            content.push_str(&hook_mainder);
+                        }
+                    }
+
                     // NOTE
                     // This should come later!!
+                    // because pipe should respect all other macro attributes 
+                    // not the other way
                     if frag.pipe {
                         self.state.pipe_value = content;
                         lexor.escape_next_newline();
@@ -1178,6 +1372,8 @@ impl Processor {
             EvalResult::InvalidArg => {
                 if self.state.strict {
                     return Err(RadError::StrictPanic);
+                } else {
+                    remainder.push_str(&frag.whole_string);
                 }
             }
             EvalResult::InvalidName =>  { // Failed to invoke
@@ -1218,6 +1414,20 @@ impl Processor {
     // ----------
     // Start of miscellaenous methods
     // <MISC>
+    /// Get comment chararacter
+    ///
+    /// This will return custom character if existent
+    pub(crate) fn get_comment_char(&self) -> char {
+        comment_start(self.state.comment_char)
+    }
+
+    /// Get macro chararacter
+    ///
+    /// This will return custom character if existent
+    pub(crate) fn get_macro_char(&self) -> char {
+        macro_start(self.state.macro_char)
+    }
+
     /// Get mutable reference of macro map
     pub(crate) fn get_map(&mut self) -> &mut MacroMap {
         &mut self.map
@@ -1270,7 +1480,7 @@ impl Processor {
     }
 
     /// Recover backup information into the processor
-    fn recover(&mut self, backup: SandboxBackup) {
+    fn recover(&mut self, backup: SandboxBackup) -> Result<(), RadError> {
         // NOTE ::: Set file should come first becuase set_file override line number and character number
         self.logger.set_file(&backup.current_input);
         self.state.current_input = backup.current_input;
@@ -1278,7 +1488,8 @@ impl Processor {
         self.logger.recover_lines(backup.logger_lines);
 
         // Also recover env values
-        self.set_file_env(&self.state.current_input);
+        self.set_file_env(&self.state.current_input)?;
+        Ok(())
     }
 
     pub(crate) fn track_assertion(&mut self, success: bool) -> Result<(), RadError> {
@@ -1307,16 +1518,17 @@ impl Processor {
         } else {
             self.state.current_input = file.to_owned();
             self.logger.set_file(file);
-            self.set_file_env(file);
+            self.set_file_env(file)?;
             Ok(())
         }
     }
 
     /// Set some useful env values
-    fn set_file_env(&self, file: &str) {
+    fn set_file_env(&self, file: &str) -> Result<(), RadError> {
         let path = Path::new(file);
         std::env::set_var("RAD_FILE", file);
-        std::env::set_var("RAD_FILE_DIR", path.parent().unwrap().to_str().unwrap());
+        std::env::set_var("RAD_FILE_DIR", std::fs::canonicalize(path)?.parent().unwrap());
+        Ok(())
     }
 
     /// Set input as string not as &path
